'use strict';

var PATH   = require('path');

var Yeoman = require('yeoman-generator');
var Chalk  = require('chalk');
var mkdirp = require('mkdirp');
var yosay  = require('yosay');
var git    = require('gitconfiglocal');

module.exports = Yeoman.generators.Base.extend({

  constructor: function(){
    Yeoman.Base.apply(this, arguments);
    this.log(Chalk.yellow("initializing this awesome generator"));
    this.argument('appname', { type: String, required: false });
    this.appname = this.appname || ''
  },

  initializing: function() {
    var done = this.async();
    this.LOCALS = {};
    this.gitc = git('./', function(err, config){
      var url = config.remote.origin.url;
      this.gitName = url.split('/').pop();
      this.gitName = this.gitName.split('.').slice(0,-1).join('.') || this.gitName + "";
      done();
    }.bind(this));
  },

  askFor: function() {
    var done = this.async();
    var prompts = [];
    this.log(Chalk.yellow("G4 modules generator"));
    prompts.push({
      type    :'input',
      name    :'moduleName',
      message :'Module name: ',
      default : this.appname
    });
    prompts.push({
      type    :'input',
      name    :'description',
      message :'Description: '
    });
    prompts.push({
      type    :'input',
      name    :'version',
      message :'Version: ',
      default : '1.0.0',
      validate: function(str) {
        var validVersion = str.match(/^(\d+\.)(\d+\.)(\d+)$/);
        if (validVersion) {
          return true;
        } else {
          return "Please enter a valid version number ( 0.0.0 )"
        }
      }
    });
    prompts.push({
      type    :'input',
      name    :'author',
      message :'Author: ',
      store   :true
    });
    prompts.push({
      type: "list",
      name: "project",
      message: "Project",
      choices: [
        "runtime","sdk","runtime-api","sdk-api"
      ]
    });
    this.prompt(prompts, function(answers){
      console.log(answers);
      this.LOCALS         = answers;
      this.LOCALS.pckName = "@intelisis/g4-"+this.LOCALS.moduleName;
      this.LOCALS.repo    = "https://github.com/IntelisisG4/"+this.gitName+"/g4-"+this.LOCALS.project+"-"+this.LOCALS.moduleName;
      this.LOCALS.path    = this.LOCALS.project.indexOf('api') > 0 ? "back" : "front";
      done();
    }.bind(this));
  },

  writing: {
    makeAppFolder: function(){
      mkdirp.sync(this.LOCALS.moduleName);
    },
    packageJSON: function() {
      this.fs.copyTpl(
        this.templatePath('_'+this.LOCALS.path+'package.json'),
        this.destinationPath(this.LOCALS.moduleName+'/package.json'),
        this.LOCALS
      )
    },
    app: function() {
      this.destinationRoot(this.LOCALS.moduleName);
      this.sourceRoot(this.sourceRoot()+'/'+this.LOCALS.path);
    },
    bulk: function() {
      this.fs.copy(
        this.templatePath('**/*'),
        this.destinationRoot()
      )
    },
  },

  installing: function() {
    this.installDependencies({
      skipInstall :true,
      bower       :false,
      callback    : function() {
        console.log('Ready');
      }.bind(this)
    });
  }
})

