chai   = require 'chai'
expect = chai.expect
drool  = require 'drool'
path   = require 'path'
config =
  chromeOptions: 'no-sandbox'
By     = require('selenium-webdriver').By

# start server for drool test
express = require 'express'
server = express()
chaiWebdriver = require 'chai-webdriver'

# Path is npm-frontend-boilerplate and /test/fixtures/html
server.use express.static(process.cwd())
server.use express.static(process.cwd() + '/test/fixtures/html')

describe 'memory tests', ->
  beforeEach ->

    @results = []
    @driver = drool.start(config)

  afterEach ->
    return @driver.quit()

  before 'server working', (done) ->
    server.listen 9002, ->
      console.info '[INFO]', 'running on port:', 9002
      done()

  it 'inputs should not leak when added and removed', (done) ->
    return drool.flow
      setup: =>
        @driver.get 'http://localhost:9002/test/fixtures/html/inputs.html'
      action: =>
        @driver.findElement(drool.webdriver.By.css('#add-remove')).click()
      assert: (after, initial) ->
        expect(initial.counts.nodes).to.equal(after.counts.nodes, 'node count should match')
        done()
    , @driver

  it 'shows leaks', ->
    return drool.flow
      setup: =>
        @driver.get 'http://localhost:9002/test/fixtures/html/leaking.html'
      action: =>
        @driver.findElement(drool.webdriver.By.css('#leak')).click()
      assert: (after, initial) ->
        expect(initial.counts.nodes).to.not.equal(after.counts.nodes, 'node count should not match')
    , @driver

  it 'gcs correctly', ->
    return drool.flow
      setup: =>
        @driver.get 'http://localhost:9002/test/fixtures/html/leaking.html'
      action: =>
        @driver.findElement(drool.webdriver.By.css('#leak')).click()
      beforeAssert: =>
        @driver.findElement(drool.webdriver.By.css('#clean')).click()
      assert: (after, initial) ->
        # This is a hack, ony because we want to test clearing leaks
        # and since the action is run once to prewarm the cache
        expect(initial.counts.nodes - 1).to.equal(after.counts.nodes, 'node count does not grow')
    , @driver

describe 'marionette test', ->
  beforeEach ->
    @results = []
    @driver = drool.start(config)
    chai.use chaiWebdriver(@driver)
    expect = chai.expect

  afterEach ->
    return @driver.quit()

  it 'should append new div', (done) ->
    return drool.flow
      setup: =>
        @driver.get 'http://localhost:9002/test/fixtures/html/marionette.html'
      action: =>
        @driver.findElement(By.css('.test-button-marionette')).click()
        expect('#after-click-marionette').dom.to.have.text 'Test'
        @driver.executeScript "$('#fixtures').empty()"
        @driver.sleep(100)
      assert: (after, initial) ->
        expect(initial.counts.nodes).to.equal(after.counts.nodes, 'node count should match')
        done()
    , @driver

describe 'drool.flow', ->
  it 'throws helpfully when no driver is given', (done) ->
    try
      drool.flow
        setup: ->
        action: ->
        beforeAssert: ->
        assert: ->
    catch e
      expect(e.message).to.equal(
        'Please provide a driver (as returned by drool.start) as the second argument to drool.flow',
        'helpful error is not thrown'
      )
      done()

describe 'checkWd', ->
  beforeEach ->
    @driver = drool.start(config)

  afterEach ->
    return @driver.quit()
  it 'should check if webdriver exists', (done) ->
    return drool.flow
      repeatCount:0
      setup: =>
        @driver.get 'http://localhost:9002/test/fixtures/html/inputs.html'

      assert: (after, initial) ->
        expect(initial.counts.nodes).to.equal(after.counts.nodes, 'node count should match')
        done()
    , @driver
